import React, { useState, useCallback, useMemo } from 'react';
import Card from './components/Card';
import TierCard from './components/TierCard';
import HistoryList from './components/HistoryList';

// --- Scratch Card Game Types and State ---
export type Tier = {
  name: string;
  unlockPrice: number;
  buyPrice: number;
  gubblePointChance: number;
};

const TIERS: Tier[] = [
  { name: 'Start', unlockPrice: 0, buyPrice: 1, gubblePointChance: 0.0 },
  { name: 'Bronze', unlockPrice: 2, buyPrice: 3, gubblePointChance: 0.0 },
  { name: 'Silver', unlockPrice: 5, buyPrice: 7, gubblePointChance: 0.0 },
  { name: 'Gold', unlockPrice: 10, buyPrice: 15, gubblePointChance: 0.1 },
  { name: 'Platinum', unlockPrice: 20, buyPrice: 30, gubblePointChance: 0.1 },
  { name: 'Diamond', unlockPrice: 50, buyPrice: 75, gubblePointChance: 0.2 },
  { name: 'Master', unlockPrice: 100, buyPrice: 150, gubblePointChance: 0.2 },
  { name: 'Legend', unlockPrice: 200, buyPrice: 300, gubblePointChance: 0.3 },
  { name: 'Ultimate', unlockPrice: 500, buyPrice: 750, gubblePointChance: 0.3 },
  { name: 'Champion', unlockPrice: 1000, buyPrice: 1500, gubblePointChance: 0.4 },
  { name: 'Supreme', unlockPrice: 2000, buyPrice: 3000, gubblePointChance: 0.4 },
];

function App() {
  // --- Game State ---
  const [money, setMoney] = useState(20); // User starts with $20
  const [unlockedTiers, setUnlockedTiers] = useState<string[]>([]); // Unlocked tier names
  const [gubblePoints, setGubblePoints] = useState(0);
  const [card, setCard] = useState<{
    winningNumbers: number[];
    fields: { value: number; scratched: boolean; won: number | null }[];
    tier: Tier | null;
  } | null>(null);
  const [history, setHistory] = useState<{
    tier: string;
    winnings: number;
    winningNumbers: number[];
    fields: number[];
  }[]>([]);

  // --- Tier Unlock Logic ---
  const handleUnlockTier = useCallback((tier: Tier) => {
    setMoney(m => {
      if (m < tier.unlockPrice) return m;
      setUnlockedTiers(prev => prev.includes(tier.name) ? prev : [...prev, tier.name]);
      return m - tier.unlockPrice;
    });
  }, []);

  // --- Buy Card Logic ---
  const handleBuyCard = useCallback((tier: Tier) => {
    setMoney(m => {
      if (m < tier.buyPrice) return m;
      if (!unlockedTiers.includes(tier.name)) return m;
      // Generate 3 unique winning numbers (1-9)
      const winningNumbers: number[] = [];
      while (winningNumbers.length < 3) {
        const n = Math.floor(Math.random() * 9) + 1;
        if (!winningNumbers.includes(n)) winningNumbers.push(n);
      }
      // Generate 6 random fields (1-9)
      const fields = Array.from({ length: 6 }, () => ({ value: Math.floor(Math.random() * 9) + 1, scratched: false, won: null }));
      setCard({ winningNumbers, fields, tier });
      setHistory(h => [
        { tier: tier.name, winnings: 0, winningNumbers, fields: fields.map(f => f.value) },
        ...h.slice(0, 9)
      ]);
      return m - tier.buyPrice;
    });
  }, [unlockedTiers]);

  // --- Update history winnings after all fields scratched ---
  // This effect only runs when card changes
  // Use useCallback for setHistory logic
  const updateHistoryWinnings = useCallback(() => {
    if (!card) return;
    const allScratched = card.fields.every(f => f.scratched);
    if (allScratched) {
      setHistory(h => {
        const [latest, ...rest] = h;
        if (!latest || latest.winningNumbers !== card.winningNumbers) return h;
        return [
          { ...latest, winnings: card.fields.reduce((sum, f) => sum + (f.won || 0), 0) },
          ...rest
        ];
      });
    }
  }, [card]);

  // Only run when card changes
  // eslint-disable-next-line react-hooks/exhaustive-deps
  React.useEffect(updateHistoryWinnings, [card]);

  // --- Memoized winnings for current card ---
  const currentWinnings = useMemo(() => card ? card.fields.reduce((sum, f) => sum + (f.won || 0), 0) : 0, [card]);

  // --- UI ---
  return (
    <div className="min-h-screen bg-gray-100 flex flex-col items-center justify-center">
      <h1 className="text-3xl font-bold text-blue-600 mb-2">Scratch Card Game</h1>
      <div className="mb-4 text-lg">Money: <span className="font-mono font-bold">${money}</span></div>
      <div className="mb-4 text-lg">Gubble Points: <span className="font-mono font-bold">{gubblePoints}</span></div>
      <div className="flex flex-wrap gap-6 mb-8 justify-center">
        {TIERS.map((tier) => (
          <TierCard
            key={tier.name}
            tier={tier}
            unlocked={unlockedTiers.includes(tier.name)}
            money={money}
            onUnlock={handleUnlockTier}
            onBuy={handleBuyCard}
          />
        ))}
      </div>
      {card && (
        <>
          <div className="mb-2 text-sm text-gray-600">
            <span className="font-semibold">Winnings this card: </span>
            ${currentWinnings}
          </div>
          <Card card={card} setCard={setCard} setMoney={setMoney} setGubblePoints={setGubblePoints} />
        </>
      )}
      <HistoryList history={history} />
    </div>
  );
}

export default App;
